/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2003 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.conf.obj;

import java.security.acl.*;
import java.util.*;
import com.silverwrist.dynamo.except.*;
import com.silverwrist.dynamo.iface.*;
import com.silverwrist.dynamo.obj.*;
import com.silverwrist.venice.iface.*;
import com.silverwrist.venice.conf.iface.*;

public class DynamicImplConference extends DynamicObjectImpl
{
  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  private static DynamicClass s_baseclass = null;

  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private VeniceConference m_impl;
  private DynamicImplNamedObject m_namedobject;
  private DynamicImplSecureObjectStore m_secstore;

  /*--------------------------------------------------------------------------------
   * Constructors
   *--------------------------------------------------------------------------------
   */

  public DynamicImplConference(VeniceConference impl, DynamicClass dclass)
  {
    super(dclass);
    m_impl = impl;
    m_namedobject = new DynamicImplNamedObject(impl);
    m_secstore = new DynamicImplSecureObjectStore(impl);

  } // end constructor

  public DynamicImplConference(VeniceConference impl)
  {
    super(getBaseDClass());
    m_impl = impl;
    m_namedobject = new DynamicImplNamedObject(impl);
    m_secstore = new DynamicImplSecureObjectStore(impl);

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Overrides from class DynamicObjectImpl
   *--------------------------------------------------------------------------------
   */

  public Object get(String property_name) throws DynamicObjectException
  {
    if (m_namedobject.canGet(property_name))
      return m_namedobject.get(property_name);
    if (property_name.equals("confID"))
      return new Integer(m_impl.getConfID());
    if (property_name.equals("createdDate"))
      return m_impl.getCreatedDate();
    if (property_name.equals("lastUpdateDate"))
      return m_impl.getLastUpdateDate();
    if (property_name.equals("hostsGID"))
      return new Integer(m_impl.getHostsGID());

    try
    { // protect these property gets within a try-catch to properly process exceptions
      if (property_name.equals("hosts"))
	return m_impl.getHosts();
      if (property_name.equals("acl"))
	return m_impl.getAcl();
      if (property_name.equals("aliases"))
	return m_impl.getAliases();
      if (property_name.equals("primaryAlias"))
	return m_impl.getPrimaryAlias();

    } // end try
    catch (DatabaseException e)
    { // catch and turn into a call failure
      throw callFailed(e);

    } // end catch
    catch (AclNotFoundException e)
    { // catch and turn into a call failure
      throw callFailed(e);

    } // end catch

    return super.get(property_name);

  } // end get

  public Object call(String method_name, Object[] parameters) throws DynamicObjectException
  {
    if (m_namedobject.canCall(method_name))
      return m_namedobject.call(method_name,parameters);
    if (m_secstore.canCall(method_name))
      return m_secstore.call(method_name,parameters);

    if (method_name.equals("getConfID"))
    { // verify parameters and call method
      verifyParameterLength(method_name,parameters,0);
      return new Integer(m_impl.getConfID());

    } // end if

    if (method_name.equals("setName"))
    { // verify parameters and call method
      try
      { // make sure and catch exceptions
	verifyParameterLength(method_name,parameters,2);
	verifyParameterType(method_name,parameters,0,DynamoUser.class);
	verifyParameterType(method_name,parameters,1,String.class);
	m_impl.setName((DynamoUser)(parameters[0]),(String)(parameters[1]));
	return null;

      } // end try
      catch (DatabaseException e)
      { // catch and turn into a call failure
	throw callFailed(e);

      } // end catch
      catch (DynamoSecurityException e)
      { // catch and turn into a call failure
	throw callFailed(e);

      } // end catch

    } // end if

    if (method_name.equals("getCreatedDate"))
    { // verify parameters and call method
      verifyParameterLength(method_name,parameters,0);
      return m_impl.getCreatedDate();

    } // end if

    if (method_name.equals("getLastUpdateDate"))
    { // verify parameters and call method
      verifyParameterLength(method_name,parameters,0);
      return m_impl.getLastUpdateDate();

    } // end if

    if (method_name.equals("setLastUpdateDate"))
    { // verify parameters and call method
      try
      { // make sure and catch exceptions
	verifyParameterLength(method_name,parameters,2);
	verifyParameterType(method_name,parameters,0,DynamoUser.class);
	verifyParameterType(method_name,parameters,1,java.util.Date.class);
	m_impl.setLastUpdateDate((DynamoUser)(parameters[0]),(java.util.Date)(parameters[1]));
	return null;

      } // end try
      catch (DatabaseException e)
      { // catch and turn into a call failure
	throw callFailed(e);

      } // end catch
      catch (DynamoSecurityException e)
      { // catch and turn into a call failure
	throw callFailed(e);

      } // end catch

    } // end if

    if (method_name.equals("getHostsGID"))
    { // verify parameters and call method
      verifyParameterLength(method_name,parameters,0);
      return new Integer(m_impl.getHostsGID());

    } // end if

    if (method_name.equals("getHosts"))
    { // verify parameters and call method
      try
      { // make sure and catch exceptions
	verifyParameterLength(method_name,parameters,0);
	return m_impl.getHosts();

      } // end try
      catch (DatabaseException e)
      { // catch and turn into a call failure
	throw callFailed(e);

      } // end catch

    } // end if

    if (method_name.equals("getAcl"))
    { // verify parameters and call method
      try
      { // make sure and catch exceptions
	verifyParameterLength(method_name,parameters,0);
	return m_impl.getAcl();

      } // end try
      catch (DatabaseException e)
      { // catch and turn into a call failure
	throw callFailed(e);

      } // end catch
      catch (AclNotFoundException e)
      { // catch and turn into a call failure
	throw callFailed(e);

      } // end catch

    } // end if

    if (method_name.equals("getAliases"))
    { // verify parameters and call method
      try
      { // make sure and catch exceptions
	verifyParameterLength(method_name,parameters,0);
	return m_impl.getAliases();

      } // end try
      catch (DatabaseException e)
      { // catch and turn into a call failure
	throw callFailed(e);

      } // end catch

    } // end if

    if (method_name.equals("getPrimaryAlias"))
    { // verify parameters and call method
      try
      { // make sure and catch exceptions
	verifyParameterLength(method_name,parameters,0);
	return m_impl.getPrimaryAlias();

      } // end try
      catch (DatabaseException e)
      { // catch and turn into a call failure
	throw callFailed(e);

      } // end catch

    } // end if

    if (method_name.equals("setPrimaryAlias"))
    { // verify parameters and call method
      try
      { // make sure and catch exceptions
	verifyParameterLength(method_name,parameters,2);
	verifyParameterType(method_name,parameters,0,DynamoUser.class);
	verifyParameterType(method_name,parameters,1,String.class);
	m_impl.setPrimaryAlias((DynamoUser)(parameters[0]),(String)(parameters[1]));
	return null;

      } // end try
      catch (DatabaseException e)
      { // catch and turn into a call failure
	throw callFailed(e);

      } // end catch
      catch (DynamoSecurityException e)
      { // catch and turn into a call failure
	throw callFailed(e);

      } // end catch

    } // end if

    if (method_name.equals("addAlias"))
    { // verify parameters and call method
      try
      { // make sure and catch exceptions
	verifyParameterLength(method_name,parameters,2);
	verifyParameterType(method_name,parameters,0,DynamoUser.class);
	verifyParameterType(method_name,parameters,1,String.class);
	m_impl.addAlias((DynamoUser)(parameters[0]),(String)(parameters[1]));
	return null;

      } // end try
      catch (DatabaseException e)
      { // catch and turn into a call failure
	throw callFailed(e);

      } // end catch
      catch (DynamoSecurityException e)
      { // catch and turn into a call failure
	throw callFailed(e);

      } // end catch

    } // end if

    if (method_name.equals("removeAlias"))
    { // verify parameters and call method
      try
      { // make sure and catch exceptions
	verifyParameterLength(method_name,parameters,2);
	verifyParameterType(method_name,parameters,0,DynamoUser.class);
	verifyParameterType(method_name,parameters,1,String.class);
	m_impl.removeAlias((DynamoUser)(parameters[0]),(String)(parameters[1]));
	return null;

      } // end try
      catch (DatabaseException e)
      { // catch and turn into a call failure
	throw callFailed(e);

      } // end catch
      catch (DynamoSecurityException e)
      { // catch and turn into a call failure
	throw callFailed(e);

      } // end catch

    } // end if

    return super.call(method_name,parameters);

  } // end call

  /*--------------------------------------------------------------------------------
   * External static operations
   *--------------------------------------------------------------------------------
   */

  public static synchronized DynamicClass getBaseDClass()
  {
    if (s_baseclass==null)
      s_baseclass = new DynamicClassConference();
    return s_baseclass;

  } // end getBaseDClass

} // end class DynamicImplConference
