/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2003 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.conf.obj;

import com.silverwrist.dynamo.except.*;
import com.silverwrist.dynamo.iface.*;
import com.silverwrist.dynamo.obj.*;
import com.silverwrist.venice.iface.*;
import com.silverwrist.venice.conf.iface.*;

public class DynamicImplConferenceAccess extends DynamicObjectImpl
{
  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  private static DynamicClass s_baseclass = null;

  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private ConferenceAccessObject m_impl;

  /*--------------------------------------------------------------------------------
   * Constructors
   *--------------------------------------------------------------------------------
   */

  public DynamicImplConferenceAccess(ConferenceAccessObject impl, DynamicClass dclass)
  {
    super(dclass);
    m_impl = impl;

  } // end constructor

  public DynamicImplConferenceAccess(ConferenceAccessObject impl)
  {
    super(getBaseDClass());
    m_impl = impl;

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Overrides from class DynamicObjectImpl
   *--------------------------------------------------------------------------------
   */

  public Object call(String method_name, Object[] parameters) throws DynamicObjectException
  {
    if (method_name.equals("getConferences"))
    { // call the getConferences method
      try
      { // verify the parameters and call the method
	verifyParameterLength(method_name,parameters,2);
	verifyParameterType(method_name,parameters,0,DynamoUser.class);
	verifyParameterType(method_name,parameters,1,VeniceCommunity.class);
	return m_impl.getConferences((DynamoUser)(parameters[0]),(VeniceCommunity)(parameters[0]));

      } // end try
      catch (DatabaseException e)
      { // reflect the exception back up
	throw callFailed(e);

      } // end catch

    } // end if

    if (method_name.equals("getConference"))
    { // call one of the getConference methods
      try
      { // verify the parameters and call the method
	verifyParameterLength(method_name,parameters,1);
	if ((parameters[0]==null) || (parameters[0] instanceof String))
	  return m_impl.getConference((String)(parameters[0]));
	int foo = getIntValue(method_name,parameters,0);
	return m_impl.getConference(foo);

      } // end try
      catch (DatabaseException e)
      { // reflect the exception back up
	throw callFailed(e);

      } // end catch
 
    } // end if

    if (method_name.equals("getLinkedConference"))
    { // call one of the getLinkedConference methods
      try
      { // verify the parameters and call the method
	verifyParameterLength(method_name,parameters,2);
	verifyParameterType(method_name,parameters,0,VeniceCommunity.class);
	if ((parameters[1]==null) || (parameters[1] instanceof String))
	  return m_impl.getLinkedConference((VeniceCommunity)(parameters[0]),(String)(parameters[1]));
	int foo = getIntValue(method_name,parameters,1);
	return m_impl.getLinkedConference((VeniceCommunity)(parameters[0]),foo);

      } // end try
      catch (DatabaseException e)
      { // reflect the exception back up
	throw callFailed(e);

      } // end catch
 
    } // end if

    return super.call(method_name,parameters);

  } // end call

  /*--------------------------------------------------------------------------------
   * External static operations
   *--------------------------------------------------------------------------------
   */

  public static synchronized DynamicClass getBaseDClass()
  {
    if (s_baseclass==null)
      s_baseclass = new DynamicClassConferenceAccess();
    return s_baseclass;

  } // end getBaseDClass

} // end class DynamicImplConferenceAccess
