/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2003 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.dynamo.dialog;

import java.util.*;
import org.w3c.dom.Element;
import com.silverwrist.util.*;
import com.silverwrist.dynamo.except.DialogException;

class LanguageListField extends PickListField
{
  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  private static Language UNKNOWN_LANGUAGE = International.get().getLanguageForCode("i-default");

  /*--------------------------------------------------------------------------------
   * Constructors
   *--------------------------------------------------------------------------------
   */

  LanguageListField(Element elt) throws DialogException
  {
    super(elt);

  } // end constructor

  protected LanguageListField(LanguageListField other)
  {
    super(other);

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Abstract implementations from class BaseDialogField
   *--------------------------------------------------------------------------------
   */

  public Object clone()
  {
    return new LanguageListField(this);

  } // end clone

  /*--------------------------------------------------------------------------------
   * Overrides from class BaseDialogField
   *--------------------------------------------------------------------------------
   */

  protected boolean isNull(Object value)
  {
    return ((value==null) || UNKNOWN_LANGUAGE.equals(value));

  } // end isNull

  /*--------------------------------------------------------------------------------
   * Abstract implementations from class PickListField
   *--------------------------------------------------------------------------------
   */

  protected Iterator getChoices()
  {
    return International.get().getLanguageList().iterator();

  } // end getChoices

  protected String getChoiceName(Object choice)
  {
    return ((Language)choice).getCode();

  } // end getChoiceName

  protected String getChoiceDisplay(Object choice)
  {
    return ((Language)choice).getName();

  } // end getChoiceDisplay

  protected Object getChoiceForName(String name)
  {
    return International.get().getLanguageForCode(name);

  } // end getChoiceForName

  protected boolean isValidChoice(Object choice)
  {
    if (!(choice instanceof Language))
      return false;
    Language x = International.get().getLanguageForCode(((Language)choice).getCode());
    return x.equals(choice);

  } // end isValidChoice

  /*--------------------------------------------------------------------------------
   * Overrides from class PickListField
   *--------------------------------------------------------------------------------
   */

  public boolean containsValue()
  {
    Object v = getValue();
    return ((v!=null) && !UNKNOWN_LANGUAGE.equals(v));

  } // end containsValue

} // end class LanguageListField
