/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2003 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.dynamo.dict;

import com.silverwrist.dynamo.Namespaces;
import com.silverwrist.dynamo.except.*;
import com.silverwrist.dynamo.htmlcheck.MarkupData;
import com.silverwrist.dynamo.iface.*;
import com.silverwrist.dynamo.util.*;

class Spellchecker implements HTMLRewriter
{
  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  private static QualifiedNameKey NAME = new QualifiedNameKey(Namespaces.SPELLCHECKER_PROPERTIES_NAMESPACE,"errors");

  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private DynamoDictionary[] m_dicts;
  private String m_begin_error;
  private String m_end_error;

  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  Spellchecker(DynamoDictionary[] dicts, String begin_error, String end_error)
  {
    m_dicts = dicts;
    m_begin_error = begin_error;
    m_end_error = end_error;

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Implementations from interface HTMLRewriter
   *--------------------------------------------------------------------------------
   */

  public QualifiedNameKey getName()
  {
    return NAME;

  } // end getName

  public MarkupData rewrite(String data, String prefix, String suffix, HTMLRewriterSite site)
  {
    for (int i=0; i<m_dicts.length; i++)
    { // check word against all the dictionaries we have
      if (m_dicts[i].check(data))
	return null;

    } // end for

    // not found - return word with error highlighted
    return new MarkupData(prefix,m_begin_error,data,m_end_error,suffix);

  } // end rewrite

} // end class Spellchecker
