/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2002 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.dynamo.event;

import java.util.EventObject;

/**
 * The root class from which all Dynamo event state objects are derived.
 * <p> All events are constructed with a reference to the object, the "source", that is logically deemed
 * to be the object upon which the event in question initially occurred upon.
 *
 * @author Eric J. Bowersox &lt;erbo@silcom.com&gt;
 * @version X
 */
public class DynamoEventObject extends EventObject
{
  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  /**
   * Constructs a new <CODE>DynamoEventObject</CODE>.
   *
   * @param src The object on which the event initially occurred.
   */
  public DynamoEventObject(Object src)
  {
    super(src);

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Overrides from class EventObject
   *--------------------------------------------------------------------------------
   */

  /**
   * Returns a string representation of the object. In general, the <CODE>toString</CODE> method returns a string
   * that "textually represents" this object. 
   *
   * @return A string representation of the object.
   */
  public String toString()
  {
    return "DynamoEventObject: source = " + source;

  } // end toString

} // end class DynamoEventObject
