/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2002-03 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.dynamo.except;

import java.text.MessageFormat;
import java.util.*;
import org.w3c.dom.*;
import com.silverwrist.util.xml.XMLLoadException;

public class ConfigException extends ExternalException
{
  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private Node m_locus = null;    // locus in configuration of exception

  /*--------------------------------------------------------------------------------
   * Constructors
   *--------------------------------------------------------------------------------
   */

  public ConfigException(XMLLoadException xle)
  {
    super(ConfigException.class,"DynamoExceptionMessages","configException.xmlLoadException",xle);
    setParameter(0,xle.getMessage());
    m_locus = xle.getLocus();

  } // end constructor

  public ConfigException(ModuleException me)
  {
    super(ConfigException.class,"DynamoExceptionMessages","configException.moduleException",me);
    setParameter(0,me.getMessage());

  } // end constructor

  public ConfigException(Class caller, String bundle, String message_id)
  {
    super(caller,bundle,message_id);

  } // end constructor

  public ConfigException(Class caller, String bundle, String message_id, Throwable inner)
  {
    super(caller,bundle,message_id,inner);

  } // end constructor

  public ConfigException(Class caller, String bundle, String message_id, Node locus)
  {
    super(caller,bundle,message_id);
    m_locus = locus;

  } // end constructor

  public ConfigException(Class caller, String bundle, String message_id, Throwable inner, Node locus)
  {
    super(caller,bundle,message_id,inner);
    m_locus = locus;

  } // end constructor

  /*--------------------------------------------------------------------------------
   * External operations
   *--------------------------------------------------------------------------------
   */

  public Node getLocus()
  {
    return m_locus;

  } // end getLocus

} // end class ConfigException
