/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2002-03 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.dynamo.iface;

/**
 * An interface to read audit records as stored in the database.
 *
 * @author Eric J. Bowersox &lt;erbo@silcom.com&gt;
 * @version X
 */
public interface AuditRecord
{
  /**
   * Get the record number associated with this audit record.
   *
   * @return The record number of the audit record.
   */
  public long getRecordNum();

  /**
   * Get the date and time stamp at which this audit record was generated.
   *
   * @return The date and time stamp of the audit record.
   */
  public java.util.Date getTimestamp();

  /**
   * Get the namespace associated with this event.  A Dynamo namespace is generally a URI.
   *
   * @return The namespace associated with this audit record.
   */
  public String getEventNamespace();

  /**
   * Get the name associated with this event, relative to the event's namespace.
   *
   * @return The event name associated with this audit record.
   */
  public String getEventName();

  /**
   * Get the user object associated with this audit record.  This is usually the user who performed the operation.
   *
   * @return The user associated with the audit record.
   */
  public DynamoUser getUser();

  /**
   * Get the subcontext identifier associated with this audit record.
   *
   * @return The subcontext associated with the audit record.
   */
  public int getSubID();

  /**
   * Get the IP address associated with this audit record.  This is usually the address of the person that
   * performed the operation.
   *
   * @return The IP address associated with the audit record.
   */
  public String getIPAddress();

  /**
   * Get a property associated with the audit record.  A property is like an "argument" to the audit record,
   * and may be of any type that can be serialized by the Dynamo PropertySerializer.
   *
   * @param index The index of the property to retrieve (0-based).
   * @return The associated property value.
   */
  public Object getProperty(int index);
  
} // end interface AuditRecord
