/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2002-03 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.dynamo.iface;

import java.util.Date;
import com.silverwrist.dynamo.except.DatabaseException;

/**
 * An interface which is used to retrieve audit records from the database by date and time.
 *
 * @author Eric J. Bowersox &lt;erbo@silcom.com&gt;
 * @version X
 */
public interface AuditRecordRetrieval
{
  /**
   * Retrieves a set of audit records from the database.
   *
   * @param start_date Starting date/time of the audit records to retrieve.
   * @param end_date Ending date/time of the audit records to retrieve.
   * @return An {@link com.silverwrist.dynamo.iface.AuditRecordSet} containing the audit records.
   * @exception com.silverwrist.dynamo.except.DatabaseException If there was a problem retrieving the audit records.
   */
  public AuditRecordSet getAuditRecords(java.util.Date start_date, java.util.Date end_date)
      throws DatabaseException;

  /**
   * Retrieves a set of audit records from the database.
   *
   * @param start_date Starting date/time of the audit records to retrieve.
   * @param end_date Ending date/time of the audit records to retrieve.
   * @param subid Subidentifier that all retrieved audit records must match.
   * @return An {@link com.silverwrist.dynamo.iface.AuditRecordSet} containing the audit records.
   * @exception com.silverwrist.dynamo.except.DatabaseException If there was a problem retrieving the audit records.
   */
  public AuditRecordSet getAuditRecords(java.util.Date start_date, java.util.Date end_date, int subid)
      throws DatabaseException;

} // end interface AuditRecordRetrieval
