/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2002-03 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.dynamo.iface;

import com.silverwrist.dynamo.except.DatabaseException;

/**
 * An interface wrapping a set of audit records.  This is returned by the methods of the
 * {@link com.silverwrist.dynamo.iface.AuditRecordRetrieval AuditRecordRetrieval} interface.
 *
 * @author Eric J. Bowersox &lt;erbo@silcom.com&gt;
 * @version X
 */
public interface AuditRecordSet
{
  /**
   * Returns the number of audit records in this set.
   *
   * @return The number of audit records in this set.
   */
  public int getRecordCount();

  /**
   * Returns a specific {@link com.silverwrist.dynamo.iface.AuditRecord AuditRecord} from this set.
   *
   * @param index 0-based index of the audit record to be retrieved.
   * @return The audit record.
   * @exception com.silverwrist.dynamo.except.DatabaseException If there was a problem retrieving the audit record.
   */
  public AuditRecord getRecord(int index) throws DatabaseException;

} // end interface AuditRecordSet
