/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2002-03 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.dynamo.iface;

import com.silverwrist.dynamo.except.ConfigException;

/**
 * A service interface which allows Dynamo components or modules to register an authentication method for
 * use by the system.  Authenticators are identified by namespace URI and name.
 *
 * @author Eric J. Bowersox &lt;erbo@silcom.com&gt;
 * @version X
 */
public interface AuthenticatorRegistration extends AuthenticatorLookup
{
  /**
   * Registers an instance of an {@link com.silverwrist.dynamo.iface.Authenticator Authenticator} with the system.
   *
   * @param method_namespace Namespace URI of the authentication method to register.
   * @param method Name of the authentication method to register.
   * @param auth The <CODE>Authenticator</CODE> which implements the authentication method.
   * @return An instance of {@link com.silverwrist.dynamo.iface.ComponentShutdown ComponentShutdown} whose
   *         {@link com.silverwrist.dynamo.iface.ComponentShutdown#shutdown() shutdown()} method may be called
   *         to unregister the authenticator.
   * @exception com.silverwrist.dynamo.except.ConfigException If there was an error registering the authenticator.
   */
  public ComponentShutdown registerAuthenticator(String method_namespace, String method, Authenticator auth)
      throws ConfigException;

} // end interface AuthenticatorRegistration
