/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2003 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.dynamo.iface;

import com.silverwrist.dynamo.except.DatabaseException;

/**
 * A runtime service interface which is used to create new instances of the HTML CHecker.  Instances of the
 * HTML Checker are identified by the namespace URI and name of their profiles, which are loaded from the
 * database.
 *
 * @author Eric J. Bowersox &lt;erbo@silcom.com&gt;
 * @version X
 */
public interface HTMLCheckerService
{
  /**
   * Creates a new instance of the HTML Checker with a specified profile and returns a reference to its
   * {@link com.silverwrist.dynamo.iface.HTMLChecker HTMLChecker} interface.
   *
   * @param profile_namespace Namespace URI for the profile to use.
   * @param profile_name Name of the profile to use.
   * @return Reference to the newly-created HTML Checker object.
   * @exception com.silverwrist.dynamo.except.DatabaseException If there was an error retrieving the specified profile
   *            information from the database.
   */
  public HTMLChecker createHTMLChecker(String profile_namespace, String profile_name) throws DatabaseException;

} // end interface HTMLCheckerService
