/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2002-03 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.dynamo.iface;

import java.io.IOException;
import com.silverwrist.dynamo.except.RenderingException;

/**
 * A service interface which is used to render an object's text to an intermediate buffer and return its
 * text representation as a string.  The object must have a registered
 * {@link com.silverwrist.dynamo.iface.TextRenderer TextRenderer} for its class, or be otherwise renderable in text
 * format.
 *
 * @author Eric J. Bowersox &lt;erbo@silcom.com&gt;
 * @version X
 */
public interface RenderImmediate
{
  /**
   * Renders the specified object and returns its text equivalent.
   *
   * @param obj The object to be rendered.
   * @return The text that results from rendering the object.
   * @exception java.io.IOException If there's an I/O error rendering the object.
   * @exception com.silverwrist.dynamo.except.RenderingException If there was another problem rendering the object.
   */
  public String renderTextObject(Object obj) throws IOException, RenderingException;

} // end interface RenderImmediate
