/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2002-03 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.dynamo.iface;

import com.silverwrist.dynamo.except.ConfigException;

/**
 * An initialization service which allows renderers to be registered for specific object classes.  Renderers may be
 * registered for either normal Java classes or <EM>dynamic classes</EM> (specified by a
 * {@link com.silverwrist.dynamo.iface.DynamicClass DynamicClass} object).  This renderer will also be used for all
 * derived classes of the object class for which it is registered, unless a more-specific registration is made for a
 * class at a lower level.
 *
 * @author Eric J. Bowersox &lt;erbo@silcom.com&gt;
 * @version X
 */
public interface RendererRegistration
{
  /**
   * Register a renderer for a Java class.
   *
   * @param klass The Java class to register the renderer for.
   * @param renderer The renderer to be registered for this class.
   * @return An instance of {@link com.silverwrist.dynamo.iface.ComponentShutdown ComponentShutdown} whose
   *         {@link com.silverwrist.dynamo.iface.ComponentShutdown#shutdown() shutdown()} method may be called
   *         to unregister this renderer.
   * @exception com.silverwrist.dynamo.except.ConfigException If the renderer could not be registered.
   */
  public ComponentShutdown registerRenderer(Class klass, Renderer renderer) throws ConfigException;

  /**
   * Register a renderer for a dynamic class.
   *
   * @param dclass The dynamic class to register the renderer for.
   * @param renderer The renderer to be registered for this dynamic class.
   * @return An instance of {@link com.silverwrist.dynamo.iface.ComponentShutdown ComponentShutdown} whose
   *         {@link com.silverwrist.dynamo.iface.ComponentShutdown#shutdown() shutdown()} method may be called
   *         to unregister this renderer.
   * @exception com.silverwrist.dynamo.except.ConfigException If the renderer could not be registered.
   */
  public ComponentShutdown registerRenderer(DynamicClass dclass, Renderer renderer) throws ConfigException;
  
} // end interface RendererRegistration
