/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2003 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.dynamo.index;

import java.io.*;
import java.lang.ref.*;
import org.apache.log4j.Logger;

class DirectoryAutoCleanup extends PhantomReference
{
  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  private static Logger logger = Logger.getLogger(DirectoryAutoCleanup.class);

  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private IndexDirectoryImpl m_directory;

  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  DirectoryAutoCleanup(IndexServiceImpl service, IndexDirectoryImpl directory, ReferenceQueue rq)
  {
    super(service,rq);
    m_directory = directory;

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Overrides from class Reference
   *--------------------------------------------------------------------------------
   */

  public void clear()
  {
    m_directory = null;
    super.clear();

  } // end clear

  /*--------------------------------------------------------------------------------
   * External operations
   *--------------------------------------------------------------------------------
   */

  void cleanup()
  {
    try
    { // close the directory!
      if (m_directory!=null)
	m_directory.close();

    } // end try
    catch (IOException e)
    { // whoops!
      logger.warn("DirectoryAutoCleanup.cleanup(): close failed",e);

    } // end catch

  } // end cleanup

} // end class DirectoryAutoCleanup
