/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2002-03 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.dynamo.velocity;

import java.util.*;

public class VelocityPage implements VelocityRenderable
{
  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private String m_templ;
  private String m_mime;
  private Hashtable m_params = new Hashtable();

  /*--------------------------------------------------------------------------------
   * Constructors
   *--------------------------------------------------------------------------------
   */

  public VelocityPage(String templ)
  {
    m_templ = templ;
    m_mime = "text/html";

  } // end constructor

  public VelocityPage(String templ, String mime)
  {
    m_templ = templ;
    m_mime = mime;

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Implementations from interface VelocityParamSupplier
   *--------------------------------------------------------------------------------
   */

  /**
   * Returns the value of a parameter set on the object.
   *
   * @param key The name of the parameter to look up.
   * @return The parameter's value, or <CODE>null</CODE> if the parameter was not set.
   */
  public Object getParameter(String key)
  {
    return m_params.get(key);

  } // end getParameter

  /**
   * Returns a <CODE>java.util.Collection</CODE> of all parameter names currently defined on this object.
   *
   * @return A collection of all parameter names currently defined.
   */
  public Collection getParameterNames()
  {
    return Collections.unmodifiableSet(m_params.keySet());

  } // end getParameterNames

  /*--------------------------------------------------------------------------------
   * Implementations from interface VelocityRenderable
   *--------------------------------------------------------------------------------
   */

  /**
   * Returns the MIME type of the output to be rendered.  Usually, this will be "text/html".
   *
   * @return The MIME type of the output.
   */
  public String getMimeType()
  {
    return m_mime;

  } // end getMimeType

  /**
   * Returns the resource name of the Velocity template to be used in rendering this object.  This pathname
   * is interpreted relative to the "resource root path" specified in the Velocity renderer's configuration.
   * The template engine loads it via the standard
   * {@link com.silverwrist.dynamo.iface.ResourceProvider ResourceProvider}, and then applies the parameters
   * contained in this object to it.
   *
   * @return The resource pathname of the Velocity template to use.
   */
  public String getTemplateName()
  {
    return m_templ;

  } // end getTemplateName

  /*--------------------------------------------------------------------------------
   * External operations
   *--------------------------------------------------------------------------------
   */

  public void setMimeType(String s)
  {
    m_mime = s;

  } // end setMimeType

  public void setTemplateName(String s)
  {
    m_templ = s;

  } // end setTemplateName

  public void setParameter(String key, Object val)
  {
    m_params.put(key,val);

  } // end setParameter

} // end class VelocityPage
