/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2002 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.dynamo.test;

import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import org.apache.log4j.Logger;
import com.silverwrist.dynamo.Verb;
import com.silverwrist.dynamo.except.*;
import com.silverwrist.dynamo.iface.*;
import com.silverwrist.dynamo.servlet.ServletBase;

public class TestServlet1 extends ServletBase implements ComponentShutdown
{
  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  private static Logger logger = Logger.getLogger(TestServlet1.class);

  /*--------------------------------------------------------------------------------
   * Abstract implementations from class ServletBase
   *--------------------------------------------------------------------------------
   */

  protected Object process(Request r, Application app) throws Exception
  {
    logger.info("Processing request for TestServlet1");
    StringBuffer b = new StringBuffer("This is a test.\n");
    b.append("This is only a test.\n");
    b.append("If this had been an actual emergency, we would all be dead by now.\n");
    b.append("=================================\n");
    b.append("Context path: " + r.getContextPath() + "\n");
    b.append("Request path: " + r.getRequestPath() + "\n");
    b.append("Extra path: " + r.getExtraPath() + "\n");
    b.append("Query string: " + r.getQueryString() + "\n");
    b.append("Verb: " + r.getVerb().getName() + "\n");
    b.append("Source address: " + r.getSourceAddress() + "\n");
    b.append("Parameters:\n");
    Iterator it = r.getParameters().keySet().iterator();
    while (it.hasNext())
    { // iterate over parameters
      String name = (String)(it.next());
      b.append("    " + name + " = { ");
      String[] vals = (String[])(r.getParameters().get(name));
      for (int i=0; i<vals.length; i++)
      { // loop over values
	if (i>0)
	  b.append(", ");
	b.append(vals[i]);

      } // end for

      b.append(" }\n");

    } // end while

    return b;

  } // end process

  /*--------------------------------------------------------------------------------
   * Overrides from class ServletBase
   *--------------------------------------------------------------------------------
   */

  protected void initServlet(ServiceProvider services) throws DynamoException
  {
    logger.info("Initializing TestServlet1");

  } // end initServlet

  /*--------------------------------------------------------------------------------
   * Implementations from interface ComponentShutdown
   *--------------------------------------------------------------------------------
   */

  public void shutdown()
  {
    logger.info("Destroying TestServlet1");

  } // end shutdown

} // end class TestServlet1
