/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2003 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.community;

import com.silverwrist.dynamo.iface.*;
import com.silverwrist.dynamo.util.*;
import com.silverwrist.venice.iface.*;

public final class CommunityServiceDescriptor
{
  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private int m_index;
  private QualifiedNameKey m_qname;
  private String m_shortvar;
  private CommunityServiceController m_csc;

  /*--------------------------------------------------------------------------------
   * Constructors
   *--------------------------------------------------------------------------------
   */

  CommunityServiceDescriptor(Integer index, QualifiedNameKey qname, String shortvar, CommunityServiceController csc)
  {
    m_index = index.intValue();
    m_qname = qname;
    m_shortvar = shortvar;
    m_csc = csc;

  } // end constructor

  CommunityServiceDescriptor(int index, QualifiedNameKey qname, String shortvar, CommunityServiceController csc)
  {
    m_index = index;
    m_qname = qname;
    m_shortvar = shortvar;
    m_csc = csc;

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Package-internal operations
   *--------------------------------------------------------------------------------
   */

  CommunityServiceController getController()
  {
    return m_csc;

  } // end getController

  String getShortVar()
  {
    return m_shortvar;

  } // end getShortVar

  /*--------------------------------------------------------------------------------
   * External operations
   *--------------------------------------------------------------------------------
   */

  public int getIndex()
  {
    return m_index;

  } // end getindex

  public QualifiedNameKey getName()
  {
    return m_qname;

  } // end getName

  public String getDescription()
  {
    return m_csc.getDescription();

  } // end getDescription

  public DynamicObject getAccessObject()
  {
    return m_csc.getAccessObject();

  } // end getAccessObject

} // end class CommunityServiceDescriptor
