/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2003 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.menu;

import com.silverwrist.dynamo.db.NamespaceCache;
import com.silverwrist.dynamo.db.OpsBase;
import com.silverwrist.dynamo.except.*;
import com.silverwrist.dynamo.iface.*;

abstract class MenuDatabaseOps extends OpsBase
{
  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private NamespaceCache m_ns_cache;

  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  protected MenuDatabaseOps(DBConnectionPool pool)
  {
    super(pool);

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Internal operations
   *--------------------------------------------------------------------------------
   */

  protected final int namespaceNameToId(String name) throws DatabaseException
  {
    return m_ns_cache.namespaceNameToId(name);

  } // end namespaceNameToId

  protected final String namespaceIdToName(int id) throws DatabaseException
  {
    return m_ns_cache.namespaceIdToName(id);

  } // end namespaceIdToName

  /*--------------------------------------------------------------------------------
   * Overrides from class OpsBase
   *--------------------------------------------------------------------------------
   */

  public void dispose()
  {
    m_ns_cache = null;
    super.dispose();

  } // end dispose

  /*--------------------------------------------------------------------------------
   * Abstract operations
   *--------------------------------------------------------------------------------
   */

  abstract MenuDefinition getMenu(String namespace, String name) throws DatabaseException;

  abstract void setMenuTitle(int id, String title) throws DatabaseException;

  abstract void setMenuSubtitle(int id, String subtitle) throws DatabaseException;

  abstract void setDefaultVal(int id, String variable, String value) throws DatabaseException;

  abstract void addVariable(int id, String variable, String value) throws DatabaseException;

  abstract void removeVariable(int id, String variable) throws DatabaseException;

  abstract void setItemSequence(int id, int old_seq, int new_seq) throws DatabaseException;

  abstract void setItemEnable(int id, int seq, boolean enable) throws DatabaseException;

  abstract void setItemIndent(int id, int seq, int indent) throws DatabaseException;

  abstract void setItemText(int id, int seq, String text) throws DatabaseException;

  abstract void setItemLink(int id, int seq, String linktype, String link) throws DatabaseException;

  abstract void setItemTarget(int id, int seq, String target) throws DatabaseException;

  abstract void setItemTitle(int id, int seq, String title) throws DatabaseException;

  abstract void setItemOnClick(int id, int seq, String on_click) throws DatabaseException;

  abstract void setItemPermission(int id, int seq, String namespace, String name) throws DatabaseException;

  abstract void setItemIfDef(int id, int seq, String var) throws DatabaseException;

  abstract void setItemIfNotDef(int id, int seq, String var) throws DatabaseException;

  abstract void addItem(int id, String type, int seq, String text, String linktype, String link)
      throws DatabaseException;

  abstract void deleteItem(int id, int seq) throws DatabaseException;

  abstract void deleteMenu(int id) throws DatabaseException;

  abstract int createMenu(String namespace, String name, String title) throws DatabaseException;

  /*--------------------------------------------------------------------------------
   * External operations
   *--------------------------------------------------------------------------------
   */

  void setNamespaceCache(NamespaceCache nsc)
  {
    m_ns_cache = nsc;

  } // end setNamespaceCache

  /*--------------------------------------------------------------------------------
   * External static operations
   *--------------------------------------------------------------------------------
   */

  static MenuDatabaseOps get(DBConnectionPool pool) throws ConfigException
  {
    return (MenuDatabaseOps)get(pool,MenuDatabaseOps.class.getClassLoader(),
				MenuDatabaseOps.class.getName() + "_","MenuDatabaseOps");

  } // end get

} // end class MenuDatabaseOps
