/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2003 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.sidebox;

import java.util.*;
import com.silverwrist.dynamo.db.OpsBase;
import com.silverwrist.dynamo.except.*;
import com.silverwrist.dynamo.iface.*;
import com.silverwrist.dynamo.util.*;

abstract class SideboxOps extends OpsBase
{
  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  protected SideboxOps(DBConnectionPool pool)
  {
    super(pool);

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Abstract operations
   *--------------------------------------------------------------------------------
   */

  abstract int[] getSideboxIDList(int uid, int context_nsid, String context_name, Object context_param)
      throws DatabaseException;

  abstract PropertyKey getSideboxType(int sbid) throws DatabaseException;

  abstract PropertyKey getSideboxIdentity(int sbid) throws DatabaseException;

  abstract Object getSideboxProperty(int sbid, PropertyKey key) throws DatabaseException;

  abstract List getSideboxDescriptors(int uid, int context_nsid, String context_name, Object context_param)
      throws DatabaseException;

  abstract List getAllSideboxDescriptors(int context_nsid, String context_name) throws DatabaseException;

  abstract void moveSideboxItemToPosition(int uid, int context_nsid, String context_name, Object context_param,
					  int sbid, int new_pos) throws DatabaseException;

  abstract void removeSidebox(int uid, int context_nsid, String context_name, Object context_param, int sbid)
      throws DatabaseException;

  abstract void addSidebox(int uid, int context_nsid, String context_name, Object context_param, int sbid)
      throws DatabaseException;

  abstract void copyUserConfig(int uid_from, int uid_to) throws DatabaseException;

  /*--------------------------------------------------------------------------------
   * External static operations
   *--------------------------------------------------------------------------------
   */

  static SideboxOps get(DBConnectionPool pool) throws ConfigException
  {
    return (SideboxOps)get(pool,SideboxOps.class.getClassLoader(),SideboxOps.class.getName() + "_","SideboxOps");

  } // end get

} // end class SideboxOps
